@@ -133,6 +133,15 @@ void BacktraceGenerator::slotProcessExited(int exitCode, QProcess::ExitStatus ex
     Debugger::expandString(tmp);
 
     m_parsedBacktrace = tmp + m_parser->informationLines() + m_parser->parsedBacktrace();
+    m_sentryPayload = [this]() -> QByteArray {
+        const QString sentryPayloadFile = m_tempDirectory->path() + QLatin1String("/sentry_payload.json");
+        QFile file(sentryPayloadFile);
+        if (!file.open(QFile::ReadOnly)) {
+            qCWarning(DRKONQI_LOG) << "Could not open sentry payload file" << sentryPayloadFile;
+            return {};
+        }
+        return file.readAll();
+    }();
     m_state = Loaded;
     Q_EMIT stateChanged();
 
@@ -180,9 +189,9 @@ void BacktraceGenerator::setBackendPrepared()
     Q_EMIT starting();
 
     m_proc = new KProcess;
-    m_proc->setEnv(QStringLiteral("LC_ALL"), QStringLiteral("C")); // force C locale
+    m_proc->setEnv(QStringLiteral("LC_ALL"), QStringLiteral("C.UTF-8")); // force C locale
 
-    // Temporary directory for the preeamble.py to write data into, we can then conveniently pick it up from there.
+    // Temporary directory for the preamble.py to write data into, we can then conveniently pick it up from there.
     // Only useful for data that is not meant to appear in the trace (e.g. sentry payloads).
     if (!m_tempDirectory) {
         m_tempDirectory = std::make_unique<QTemporaryDir>();
@@ -190,9 +199,6 @@ void BacktraceGenerator::setBackendPrepared()
     if (!m_tempDirectory->isValid()) {
         qCWarning(DRKONQI_LOG) << "Failed to create temporary directory for generator!";
     } else {
-#if WITH_SENTRY
-        m_proc->setEnv(QStringLiteral("DRKONQI_WITH_SENTRY"), QStringLiteral("1"));
-#endif
         m_proc->setEnv(QStringLiteral("DRKONQI_TMP_DIR"), m_tempDirectory->path());
         m_proc->setEnv(QStringLiteral("DRKONQI_VERSION"), QStringLiteral(PROJECT_VERSION));
         m_proc->setEnv(QStringLiteral("DRKONQI_APP_VERSION"), DrKonqi::appVersion());
@@ -229,6 +235,7 @@ void BacktraceGenerator::setBackendPrepared()
     connect(m_proc, static_cast<void (KProcess::*)(int, QProcess::ExitStatus)>(&KProcess::finished), this, &BacktraceGenerator::slotProcessExited);
     connect(m_proc, &KProcess::errorOccurred, this, &BacktraceGenerator::slotOnErrorOccurred);
 
+    qCDebug(DRKONQI_LOG) << "Starting debugger" << m_proc->program() << m_proc->arguments();
     m_proc->start();
 }
 
@@ -244,11 +251,7 @@ QString BacktraceGenerator::debuggerName() const
 
 QByteArray BacktraceGenerator::sentryPayload() const
 {
-    const QString sentryPayloadFile = m_tempDirectory->path() + QLatin1String("/sentry_payload.json");
-    QFile file(sentryPayloadFile);
-    if (!file.open(QFile::ReadOnly)) {
-        qCWarning(DRKONQI_LOG) << "Could not open sentry payload file" << sentryPayloadFile;
-        return {};
-    }
-    return file.readAll();
+    return m_sentryPayload;
 };
+
+#include "moc_backtracegenerator.cpp"
