@@ -25,17 +25,18 @@ public:
     // In a function cause it is used in more than one location.
     static QByteArray keyFilename();
     static QByteArray keyPickup();
+    static QByteArray keyCursor();
 
     // Other bits and bobs
-    const QByteArray m_cursor;
-    const EntriesHash m_rawData;
+    QByteArray m_cursor;
+    EntriesHash m_rawData;
 
     // Journal Entry values
-    const uid_t uid = 0;
-    const pid_t pid = -1;
-    const QString exe;
-    const QString filename; // core dump file if available (may be /dev/null if the core is stored in journal directly - only in older systemds)
-    const QString systemd_unit;
+    uid_t uid = 0;
+    pid_t pid = -1;
+    QString exe;
+    QString filename; // core dump file if available (may be /dev/null if the core is stored in journal directly - only in older systemds)
+    QString systemd_unit;
 
 private:
     static EntriesHash documentToHash(const QJsonDocument &document);
