@@ -30,6 +30,7 @@
 #include "DumpTruckInterface.h"
 
 using namespace std::chrono_literals;
+using namespace Qt::StringLiterals;
 
 static QString drkonqiExe()
 {
@@ -37,7 +38,7 @@ static QString drkonqiExe()
     static QStringList paths = QFile::decodeName(qgetenv("LIBEXEC_PATH")).split(QLatin1Char(':'), Qt::SkipEmptyParts)
         + QStringList{
             QCoreApplication::applicationDirPath(), // then look where our application binary is located
-            QLibraryInfo::location(QLibraryInfo::LibraryExecutablesPath), // look where libexec path is (can be set in qt.conf)
+            QLibraryInfo::path(QLibraryInfo::LibraryExecutablesPath), // look where libexec path is (can be set in qt.conf)
             QFile::decodeName(KDE_INSTALL_FULL_LIBEXECDIR), // look at our installation location
         };
     static QString exec = QStandardPaths::findExecutable(QStringLiteral("drkonqi"), paths);
@@ -62,7 +63,7 @@ static ArgumentsPidTuple metadataArguments(const Coredump &dump, const QString &
 
         if (key == QLatin1String("exe")) {
             if (value.endsWith(QStringLiteral("/drkonqi"))) {
-                qWarning() << "drkonqi crashed, we aren't going to invoke it again, we might be the reason it crashd :O";
+                qWarning() << "drkonqi crashed, we aren't going to invoke it again, we might be the reason it crashed :O";
                 return {};
             }
             if (value != dump.exe) {
@@ -125,6 +126,14 @@ static bool tryDrkonqi(const Coredump &dump)
         return false;
     }
 
+    QSettings metadata(Metadata::metadataPath(dump.pid), QSettings::IniFormat);
+    metadata.beginGroup("DrKonqi"_L1);
+    if (metadata.value("PickedUp"_L1).toBool()) {
+        // already being worked on!
+        return true;
+    }
+    metadata.endGroup();
+
     QStringList arguments;
     bool foundPID = false;
 
@@ -150,12 +159,14 @@ static bool tryDrkonqi(const Coredump &dump)
     }
 
     // Append Coredump data. This allow us to not have to talk to journald again on the drkonqi side.
-    QSettings metadata(Metadata::metadataPath(dump.pid), QSettings::IniFormat);
     metadata.beginGroup(QStringLiteral("Journal"));
     for (auto it = dump.m_rawData.cbegin(); it != dump.m_rawData.cend(); ++it) {
         metadata.setValue(QString::fromUtf8(it.key()), it.value());
     }
     metadata.endGroup();
+    metadata.beginGroup("DrKonqi"_L1);
+    metadata.setValue("PickedUp"_L1, true);
+    metadata.endGroup();
     metadata.sync();
 
     setenv("DRKONQI_BACKEND", "COREDUMPD", 1);
