@@ -19,6 +19,8 @@ try:
 except ImportError:
     print("python sentry-sdk not installed :(")
 
+os.environ['LC_ALL'] = 'C.UTF-8'
+
 import gdb
 from gdb.FrameDecorator import FrameDecorator
 
@@ -38,6 +40,9 @@ import psutil
 class UnexpectedMappingException(Exception):
     pass
 
+class NoBuildIdException(Exception):
+    pass
+
 def mangle_path(path):
     if not path:
         return path
@@ -210,27 +215,73 @@ class SentryRegisters:
             return None
         return js
 
+class LockReason:
+    def __init__(self, frame: SentryFrame, type, class_name):
+        self.type = type
+        self.class_name = class_name
+        self.thread_id = gdb.selected_thread().ptid[1]
+
+    def to_dict(self):
+        return {
+            'type': self.type,
+            'class_name': self.class_name,
+            'thread_id': self.thread_id,
+            'package_name': 'java.lang',
+        }
+
+    def make(frame):
+        # export enum LockType {
+        #   LOCKED = 1,
+        #   WAITING = 2,
+        #   SLEEPING = 4,
+        #   BLOCKED = 8,
+        # }
+        func = frame.function()
+        match func:
+            case 'QtLinuxFutex::_q_futex':
+                return LockReason(frame, 8, 'QtLinuxFutex')
+            case '___pthread_cond_wait':
+                return LockReason(frame, 2, 'pthread_cond_wait')
+            case 'QWaitCondition::wait':
+                return LockReason(frame, 2, 'QWaitCondition')
+        return None
+
 class SentryTrace:
-    def __init__(self, thread):
+    def __init__(self, thread, is_crashed):
         thread.switch()
         self.frame = gdb.newest_frame()
+        self.is_crashed = is_crashed
+        self.lock_reasons = {}
+        self.was_main_thread = False
 
     def to_dict(self):
         frames = [ SentryFrame(frame) for frame in gdb.FrameIterator.FrameIterator(self.frame) ]
+        self.lock_reasons = {}
+        self.was_main_thread = False
 
-        # throw away kcrash or sigtrap frame, and above. they are useless noise
         kcrash_index = -1
         trap_index = -1
         for index, frame in enumerate(frames):
-            if frame.function() and frame.function().startswith('KCrash::defaultCrashHandler'):
-                kcrash_index = index
+            if frame.function():
+                lock_reason = LockReason.make(frame)
+                if lock_reason:
+                    r = lock_reason.to_dict()
+                    address = f'0x{str(len(self.lock_reasons.keys()))}'
+                    r['address'] = address
+                    self.lock_reasons[address] = r
+                if frame.function().startswith('KCrash::defaultCrashHandler'):
+                    kcrash_index = index
+                if frame.function().startswith('QCoreApplication::exec'):
+                    self.was_main_thread = True
             if frame.type() == gdb.SIGTRAMP_FRAME:
                 trap_index = index
         clip_index = max(kcrash_index, trap_index)
-        if clip_index > -1:
+
+        # Throw away kcrash or sigtrap frame, and above. They are useless noise - but only when on the crashing thread.
+        if self.is_crashed and clip_index > -1:
             frames = frames[(clip_index + 1):]
 
-        # Sentry format oddly wants oldest frame first. TODO
+        # Sentry format wants oldest frame first.
         frames.reverse()
         return { 'frames': [ frame.to_dict() for frame in frames ], 'registers': SentryRegisters(self.frame).to_dict() }
 
@@ -243,12 +294,40 @@ class SentryThread:
         # https://develop.sentry.dev/sdk/event-payloads/threads/
         # As per Sentry policy, the thread that crashed with an exception should not have a stack trace,
         #  but instead, the thread_id attribute should be set on the exception and Sentry will connect the two.
-        return {
+        trace = SentryTrace(self.thread, self.is_crashed)
+        # NB: trace.to_dict creates members as side effect, run it asap
+        payload = {
+            'stacktrace': trace.to_dict(),
             'id': self.thread.ptid[1],
             'name': self.thread.name,
             'crashed': self.is_crashed,
-            'stacktrace': SentryTrace(self.thread).to_dict()
+            'main': trace.was_main_thread, # side effect
+            'held_locks': trace.lock_reasons, # side effect
         }
+        # States appear not documented. They are
+        #   RUNNABLE = 'Runnable',
+        #   TIMED_WAITING = 'Timed waiting',
+        #   BLOCKED = 'Blocked',
+        #   WAITING = 'Waiting',
+        #   NEW = 'New',
+        #   TERMINATED = 'Terminated',
+        state = None
+        if self.thread.is_exited():
+            state = 'Terminated'
+        if not state:
+            for addr, reason in trace.lock_reasons.items():
+                match reason['type']:
+                    case 1:
+                        state = None # locked doesn't exist as thread state
+                    case 2:
+                        state = 'Waiting'
+                    case 4:
+                        state = 'Runnable'
+                    case 8:
+                        state = 'Blocked'
+                break
+        payload['state'] = (state or 'Runnable')
+        return payload
 
 class SentryImage:
     # NOTE: realpath hacks because neon's gdb is confused over UsrMerge symlinking of /lib to /usr/lib messing up
@@ -289,7 +368,16 @@ class SentryImage:
                     lookup = None
                 objfiles = gdb.objfiles()
                 self_objfiles = self.objfiles() # pull into scope so we have it in the trace in sentry
-                raise UnexpectedMappingException("unexpected mapping fail {} {} {} {}".format(self.file, lookup, objfiles, self_objfiles))
+                if 'sentry_sdk' in globals():
+                    progspace = gdb.selected_inferior().progspace
+                    pid_running = psutil.pid_exists(gdb.selected_inferior().pid)
+                    sentry_sdk.add_breadcrumb(
+                        category='debug',
+                        level='debug',
+                        message=f'Progspace {progspace} :: {progspace.filename} :: {progspace.is_valid()} :: pid running ({pid_running})',
+                    )
+                    sentry_sdk.capture_exception(UnexpectedMappingException("unexpected mapping fail {} {} {} {}"
+                                                                            .format(self.file, lookup, objfiles, self_objfiles)))
             return
         self.valid = True
 
@@ -298,6 +386,8 @@ class SentryImage:
         # It is the value of the build_id custom section and must be formatted
         # as UUID truncated to the leading 16 bytes.
         build_id = self.build_id()
+        if not build_id:
+            raise NoBuildIdException(f'Unexpectedly stumbled over an objfile ({self.file}) without build_id. Not creating payload.')
         truncate_bytes = 16
         build_id = build_id + ("00" * truncate_bytes)
         return str(uuid.UUID(bytes_le=binascii.unhexlify(build_id)[:truncate_bytes]))
@@ -427,6 +517,7 @@ class SentryEvent:
             ] # + SentryQMLThread().to_list(), TODO make qml more efficient it iterates everything again after the sentry threads were collected. a right waste of time!
             ,
             'event_id': uuid.uuid4().hex,
+            # Gets overwritten by ReportInterface with a more accurate value
             'timestamp': datetime.utcnow().isoformat(),
             'message': 'Signal {} in {}'.format(crash_signal, program),
             'platform': 'native',
@@ -478,7 +569,7 @@ class SentryEvent:
                                 },
                             },
                         },
-                        'stacktrace': SentryTrace(crash_thread).to_dict(),
+                        'stacktrace': SentryTrace(crash_thread, True).to_dict(),
                     }
                 ]
             }
@@ -628,9 +719,19 @@ def print_preamble():
     if thread == None:
         # Can happen when e.g. the core is missing or not readable etc. We basically aren't debugging anything
         return
+    if 'sentry_sdk' in globals():
+        sentry_sdk.add_breadcrumb(
+            category='debug',
+            level='debug',
+            message=f'Selected thread {thread}',
+        )
     # run this first as it expects the current frame to be the crashing one and qml tracing changes the frames around
     print_kcrash_error_message()
     # changes current frame and thread!
     print_qml_trace()
     # prints sentry report
-    print_sentry_payload(thread)
+    try:
+        print_sentry_payload(thread)
+    except NoBuildIdException as e:
+        print(e)
+        pass
